#!/bin/bash
ART_ETC_FOLDER=$ARTIFACTORY_HOME/etc
NODE=$(date +%s$RANDOM)
: ${ART_LOGIN:=_internal}
: ${ART_PASSWORD:=b6a50c8a15ece8753e37cbe5700bf84f}

function setPorts {
	# Artifactory's membership port, the PORT0 et PORT1 are specific to MESOS
	: ${MEMBERSHIP_PORT:=10042}
	: ${INSTANCE_PORT:=8081}
	if [ ! -z "$PORT0" ]; then
	    INSTANCE_PORT=$PORT0
	fi
	if [ ! -z "$PORT1" ]; then
	    MEMBERSHIP_PORT=$PORT1
	fi

	# Change the port
    sed -i -e 's,Connector port="\(.*\)",Connector port="'"$INSTANCE_PORT"'",g' /opt/jfrog/artifactory/tomcat/conf/server.xml
}

function setInstanceIp {
    # If no network is provided we take the first ip adress we found
    if [ -z "$ART_NETWORK" ]; then
        INSTANCE_IP=$(hostname -I|awk '{print $1}')
    # else we try to get it from the network provided
    else
        INSTANCE_IP=$(ip route show to match $ART_NETWORK | grep -Eo '[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}' | tail -1)
        if [ -z "$INSTANCE_IP" ]; then
            echo "[ERROR] Couldn't find a source IP routing to $ART_NETWORK, exiting" >&2
            exit 1
        fi
    fi
}

function setHaNodeProperties {
	local ha_home=/var/opt/jfrog/cluster
	cat >/etc/opt/jfrog/artifactory/ha-node.properties <<EOF
node.id="$NODE"
cluster.home=$ha_home
primary=false
context.url=http://$INSTANCE_IP:$INSTANCE_PORT/artifactory
membership.port=$MEMBERSHIP_PORT
EOF
}

function setPrimaryBaseUrl {
	# See how we can deduce where the PRIMARY can be reached
	# One option can be : do a plain SQL request... to the artifactory_servers...
	# Or better try to ask the MESOS DNS for that, so :
	# If the ART_PRIMARY_BASE_URL is specified we'll use that, if not we will try to resolve it
	# from mesos DNS (if we want to make it generic we need to pass a variable to specify a DNS)
	if [ -z "$ART_PRIMARY_BASE_URL" ]; then
        if [ -z "ART_PRIMARY_SERVICE_NAME" ]; then
		ART_PRIMARY_PORT=$(dig srv _instance._$ART_PRIMARY_SERVICE_NAME._tcp.marathon.mesos +short | head -1 | awk '{print $3}')
		else
		ART_PRIMARY_PORT=$(dig srv _instance._artifactory._tcp.marathon.mesos +short | head -1 | awk '{print $3}')
		fi
	    ART_PRIMARY_BASE_URL=http://artifactory.marathon.mesos:$ART_PRIMARY_PORT/artifactory
	fi
}

function waitForPrimaryNode {
	log "WAITING FOR PRIMARY NODE : $ART_PRIMARY_BASE_URL"
	local arguments=" --output /dev/null --silent --fail $ART_PRIMARY_BASE_URL/api/system/ping"
	local curl=$(curlAuth)
	until $(eval $curl $arguments)
	do
		echo "."
		sleep 2
	done
	log "PRIMARY NODE IS UP !"
}

function curlAuth {
	if [ -z "$ART_API_KEY" ]; then
		echo "curl -u$ART_LOGIN:$ART_PASSWORD"
	else
		echo "curl -H \"X-JFrog-Art-Api:$ART_API_KEY\""
	fi
}

function log {
	echo "[SECONDARY $NODE] $1"
}

function getLicenseFromPrimary {
	local arguments=" --show-error --silent --fail -X GET $ART_PRIMARY_BASE_URL/api/plugins/execute/getSecondaryLicense?params=$NODE"
	local curl=$(curlAuth)
	local response=$((eval $curl $arguments) 2>&1)
	local responseStatus=$?
	if [ $responseStatus -ne 0 ]; then
		log "Couldn't retrieve the license from the primary, got response from server $response "
		echo $responseStatus
	elif [ ! -z "$response" ]; then
		echo -n "$response" > /etc/opt/jfrog/artifactory/artifactory.lic
		echo "0"
	else
		log "License not found from primary"
		echo $responseStatus
	fi	
}

function getLicenseFromPrimaryOrDieTrying {
	local numberOfRetries=5
	local retry=0
	local sleepingTime=15
	while [ "$retry" -lt "$numberOfRetries" ]; do
		local lic=$(getLicenseFromPrimary)
		echo "return code from getLic : $lic"
		if [ "$lic" == "0" ]; then
			return 0
		fi
		retry=$((retry+1))
		log "Retry $retry getting license from primary in $sleepingTime seconds"
		sleep $sleepingTime
	done
	log "Have tried $numberOfRetries to get a license from primary without success, now exiting"
	exit 1
}

function restoreEtcFilesIfEmpty {
    #If etc folder doesn't exists OR exists but default file is missing, copies the default content from $ARTIFACTORY_HOME/defaults
    if [[ ! -d $ART_ETC_FOLDER ]] || [[ ! -e $ART_ETC_FOLDER/default ]] ; then
        echo "[WARN] $ART_ETC_FOLDER is either empty or missing default files, creating default files"
        cp -rpn $ARTIFACTORY_HOME/defaults/etc/* $ART_ETC_FOLDER/
    fi
}

function setDefaultJvmParams {
    local defaultFile=$ART_ETC_FOLDER/default
    # Change distrib type
    sed -i s/-Dartdist=rpm/-Dartdist=docker/g $defaultFile
    # In case of marathon, we can set the default Xmx with $MARATHON_APP_RESOURCE_MEM - 250M
    # We let this 250M to allow the non Heap Memory. If this wild guess is not enough, the user
    # can override all the jvm params with $JVM_OPTS
    if [ ! -z "$MARATHON_APP_RESOURCE_MEM" ]; then
        local memory=$(expr $(echo "$MARATHON_APP_RESOURCE_MEM" | cut -d'.' -f 1) - 250)
        echo "[INFO] According to Marathon memory constraint of: $MARATHON_APP_RESOURCE_MEM, will set Xmx to ${memory}m"
        local xmx="-Xms${memory}m -Xmx${memory}m"
        printf '# Adding Xmx based on Marathon memory constraint \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$xmx"'"' >> $defaultFile
    fi
    if [ ! -z "$ART_RUNTIME_OPTS" ]; then
        echo "[INFO] Will add defaults JAVA_OPTIONS to $defaultFile with : $ART_RUNTIME_OPTS"
        printf '# Adding user provided options: \nexport JAVA_OPTIONS="$JAVA_OPTIONS '"$ART_RUNTIME_OPTS"'"' >> $defaultFile
    fi
}

restoreEtcFilesIfEmpty
setDefaultJvmParams
setInstanceIp
setPorts
setHaNodeProperties
setPrimaryBaseUrl
waitForPrimaryNode
getLicenseFromPrimaryOrDieTrying

exec su artifactory -m /opt/jfrog/artifactory/tomcat/bin/catalina.sh run